public class ArrayList {
  public int size;
  public Object[] list;
  
  //@ public invariant size >= -1;
  //@ public invariant list.length > 0;
  
  public ArrayList() {
    //@ ensures size == -1 && list.length == 10;
    //@ assignable size, list;
    size = -1;
    list = new Object[10];
    
    //@ assert size == -1;
    //@ assert list.length == 10;
    //@ invariant size >= -1;
    //@ invariant list.length > 0;
  }
  
  public int size() {
    //@ ensures \result == size;
    //@ readable_if true;
    return size;
  }
  
  public Object get(int index) {
    //@ requires 0 <= index && index <= size;
    //@ ensures \result == list[index];
    //@ readable_if true;
    //@ assignable \nothing;
    if (0 <= index && index <= size) {
      return list[index];
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  public boolean contains(Object o) {
    //@ requires o != null;
    //@ ensures \result == (\exists int i; 0 <= i && i <= size; list[i] == o);
    //@ readable_if true;
    //@ assignable \nothing;
    for (int i = 0; i <= size; i++) {
      if (list[i] == o) {
        return true;
      }
    }
    return false;
  }
  
  public void set(int index, Object o) {
    //@ requires 0 <= index && index <= size;
    //@ assignable list[index];
    //@ ensures list[index] == o;
    if (0 <= index && index <= size) {
      list[index] = o;
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  private void enlarge() {
    //@ requires list.length < Integer.MAX_VALUE;
    //@ assignable list, size;
    //@ ensures list.length > \old(list.length) && size == \old(size);
    Object[] newList;
    if (list.length >= Integer.MAX_VALUE-10 && list.length < Integer.MAX_VALUE) {
      newList = new Object[Integer.MAX_VALUE];
    } else {
      newList = new Object[list.length + 10];
    }
    for (int i = 0; i <= size; i++) {
      newList[i] = list[i];
    }
    list = newList;
    
    //@ assert list.length > \old(list.length);
    //@ invariant size >= -1;
    //@ invariant list.length > 0;
  }
  
  public void add(Object o) {
    //@ requires o != null;
    //@ assignable list, size;
    //@ ensures size == \old(size) + 1 && list[size] == o;
    if (size == Integer.MAX_VALUE - 1) {
      throw new IndexOutOfBoundsException();
    }
    if (size == list.length - 1) {
      enlarge();
    }
    size = size + 1;
    list[size] = o;
    
    //@ assert size == \old(size) + 1;
    //@ assert list[size] == o;
    //@ invariant size >= -1;
    //@ invariant list.length > 0;
  }
  
  public void remove(Object o) {
    //@ requires o != null;
    //@ assignable list, size;
    //@ ensures (\exists int i; 0 <= i && i <= \old(size); list[i] == o) ==> size == \old(size) - 1;