// Prompt: I have some Java code, can you annotate this program with JML (java modelling language) Specifications.

/*@ pure @*/
public class RandomSearch {
    
    /*@ requires array != null;
      @ requires \nonnullelements(array);
      @ requires (\forall int i; 0 <= i && i < array.length-1; array[i] <= array[i+1]); // Array is sorted
      @ ensures (\result >= 0) ==> (array[\result] == value); // If result is not -1, then it must be the index of the value in the array
      @ ensures (\result == -1) ==> ((value < array[0]) || (value > array[array.length-1]) || (\forall int i; 0 <= i && i < array.length; array[i] != value)); // If result is -1, then value is not in the array
      @*/
    public static int search(int[] array, int value) {
        if (value < array[0]) return -1;
        if (value > array[array.length-1]) return -1;
        int lo = 0;
        int hi = array.length-1;
        /*@ maintaining 0 <= lo && lo <= hi && hi < array.length;
          @ maintaining (\forall int i; 0 <= i && i < lo; array[i] < value);
          @ maintaining (\forall int i; hi < i && i < array.length; array[i] > value);
          @ decreasing hi - lo;
          @*/
        while (lo <= hi) {
            int mid = lo + (hi-lo)/2;
            if (array[mid] == value) {
                return mid;
            } else if (array[mid] < value) {
                lo = mid+1;
            } else {
                hi = mid-1;
            }
        }
        return -1;
    }
}