// Toradocu/JDoctor derives annotations based on the Javadoc
// We have used the List (Java SE 8) JavaDoc as a starting point for this class
// Link to List (Java SE 8)'s JavaDoc: https://docs.oracle.com/javase/8/docs/api/java/util/List.html#contains-java.lang.Object-

public class ArrayList {
  public int size;
  public Object[] list;
  
  /** Constructs an empty list with an initial capacity of ten.
  */
  public ArrayList() {
    size = -1;
    list = new Object[10];
  }
  
  /** Returns the number of elements in this list.
    * @return the number of elements in this list
    */
  public int size() {
    return size;
  }
  
  /** Returns the element at the specified position in this list.
    * @param index index of the element to return
    * @return the element at the specified position in this list
    * @throws IndexOutOfBoundsException if the index is out of range (index < 0 || index >= size())
    */
  public Object get(int index) {
    if (0 <= index && index <= size) {
      return list[index];
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  /** Returns true if this list contains the specified object.
    * @param o object whose presence in this list is to be tested
    * @return true if this list contains the specified element
    */
  public boolean contains(Object o) {
    for (int i = 0; i <= size; i++) {
      if (list[i] == o) {
        return true;
      }
    }
    return false;
  }
  
  /** Replaces the object at the specified position in this list with the specified object.
    * @param index index of the element to replace
    * @param o object to be stored at the specified position
    * @throws IndexOutOfBoundsException if the index is out of range (index < 0 || index >= size())
    */
  public void set(int index, Object o) {
    if (0 <= index && index <= size) {
      list[index] = o;
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  /** Increases the length of the list if possible. It will increase the length by 10 if possible otherwise it will set the length to the maximum possible size.
    */
  private void enlarge() {
    Object[] newList;
    if (list.length >= Integer.MAX_VALUE-10 && list.length < Integer.MAX_VALUE) {
      newList = new Object[Integer.MAX_VALUE];
    } else {
      newList = new Object[list.length + 10];
    }
    for (int i = 0; i <= size; i++) {
      newList[i] = list[i];
    }
    list = newList;
  }
  
  /** Inserts the specified element into the list at the end.
    * @param o the object to insert
    * @throws IndexOutOfBoundsException if the list is full and its size cannot be increased
    *
    */
  public void add(Object o) {
    if (size == Integer.MAX_VALUE - 1) {
      throw new IndexOutOfBoundsException();
    }
    if (size == list.length - 1) {
      enlarge();
    }
    size = size + 1;
    list[size] = o;
  }
  
  /** Removes the first occurrence of the specified object from the list.
    * @param o the object to remove
    */
  public void remove(Object o) {
    for (int i = 0; i <= size; i++) {
      if (list[i] == o) {
        for (int j = i; j < size; j++) {
          list[j] = list[j+1];
        }
        size = size - 1;
        return;
      }
    }
  }
  
  /** Returns true if this list contains no elements.
    * @return true if this list contains no elements
    */
  public boolean isEmpty() {
    return size == -1;
  }
  
}