public class ArrayList {
  //@ public invariant -1 <= size <= Integer.MAX_VALUE;
  //@ public invariant list != null;
  //@ public invariant size < list.length;
  //@ public invariant (\forall int i; 0 <= i <= size; list[i] != null);
  public int size;
  //@ public invariant 0 <= list.length <= Integer.MAX_VALUE;
  public Object[] list;
  
  public ArrayList() {
    size = -1;
    list = new Object[10];
  }
  
  public int size() {
    return size;
  }
  
  //@ public normal_behavior
  //@   requires 0 <= index && index <= size;
  //@ also
  //@ public exceptional_behavior
  //@  requires index < 0 || index > size;
  public Object get(int index) {
    if (0 <= index && index <= size) {
      return list[index];
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  //@ requires o != null;
  public boolean contains(Object o) {
    for (int i = 0; i <= size; i++) {
      if (list[i] == o) {
        return true;
      }
    }
    return false;
  }
  
  //@ public normal_behavior
  //@   requires 0 <= index <= size;
  //@   requires o != null;
  //@   requires \type(Object) <: \elemtype(\typeof(list));
  //@ also
  //@ public exceptional_behavior
  //@   requires index < 0 || index > size;
  public void set(int index, Object o) {
    if (0 <= index && index <= size) {
      list[index] = o;
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  //@ private normal_behavior
  //@   requires list.length < Integer.MAX_VALUE;
  //@   requires \type(Object) <: \elemtype(\typeof(list));
  private void enlarge() {
    Object[] newList;
    if (list.length >= Integer.MAX_VALUE-10 && list.length < Integer.MAX_VALUE) {
      newList = new Object[Integer.MAX_VALUE];
    } else {
      newList = new Object[list.length + 10];
    }
    for (int i = 0; i <= size; i++) {
      newList[i] = list[i];
    }
    list = newList;
  }
  
  //@ public normal_behavior
  //@   requires size < Integer.MAX_VALUE - 1;
  //@   requires o != null;
  //@   requires \type(Object) <: \elemtype(\typeof(list));
  //@ also
  //@ public exceptional_behavior
  //@   requires size == Integer.MAX_VALUE - 1;
  public void add(Object o) {
    if (size == Integer.MAX_VALUE - 1) {
      throw new IndexOutOfBoundsException();
    }
    if (size == list.length - 1) {
      enlarge();
    }
    size = size + 1;
    list[size] = o;
  }
  
  //@ public normal_behavior
  //@   requires o != null && size >= 0;
  //@   requires (\exists int i; 0 <= i <= size; list[i] == o);
  //@ also
  //@ public normal_behavior
  //@   requires o != null && size >= 0;
  //@   requires (\forall int i; 0 <= i <= size; list[i] != o);
  //@ also
  //@ public normal_behavior
  //@   requires size == -1;
  public void remove(Object o) {
    for (int i = 0; i <= size; i++) {
      if (list[i] == o) {
        for (int j = i; j < size; j++) {
          list[j] = list[j+1];
        }
        size = size - 1;
        return;
      }
    }
  }
  
  public boolean isEmpty() {
    return size == -1;
  }
  

}