// The specs were inferred with Strongarm (in VM, default settings, provided by John Singleton)
// Preconditions in "//@" were provided to Strongarm.

public class Counter {
  public int value;

  public Counter() {
    value = 0;
  }
  
  //@ requires Integer.MIN_VALUE <= initialValue && initialValue <= Integer.MAX_VALUE;
  public Counter(int initialValue) {
    value = initialValue;
  }
  
  //@ requires Integer.MIN_VALUE <= this.value && this.value < Integer.MAX_VALUE;
  /*+INFERRED@
    @ public normal_behavior
    @   requires Integer.MIN_VALUE <= this.value && this.value < Integer.MAX_VALUE;
    @   ensures this.value == \old(this.value) + 1;
    @   assignable this.value;
    @*/
  public void incr() {
    value = value + 1;
  }
  
  //@ requires Integer.MIN_VALUE <= this.value && this.value <= Integer.MAX_VALUE - n;
  //@ requires n >= 0;
  /*+INFERRED@
    @ public normal_behavior
    @   requires Integer.MIN_VALUE <= this.value && this.value <= Integer.MAX_VALUE - n;
    @   requires n >= 0;
    @
    @*/
  public void incrByN(int n) {
    int tmp = n;
    while (tmp > 0) {
      incr();
      tmp = tmp - 1;
    }
  }
  
  
  //@ requires Integer.MIN_VALUE + 1 <= this.value && this.value <= Integer.MAX_VALUE;
  /*+INFERRED@
    @ public normal_behavior
    @   requires Integer.MIN_VALUE + 1 <= this.value && this.value <= Integer.MAX_VALUE;
    @   ensures this.value == \old(this.value) - 1;
    @   assignable this.value;
    @*/
  public void decr() {
    value = value - 1;
  }
  
  //@ requires Integer.MIN_VALUE + n <= this.value && this.value <= Integer.MAX_VALUE;
  //@ requires n >= 0;
  /*+INFERRED@
    @ public normal_behavior
    @   requires Integer.MIN_VALUE + n <= this.value && this.value <= Integer.MAX_VALUE;
    @   requires n >= 0;
    @
    @*/
  public void decrByN(int n) {
    int tmp = 0;
    while (tmp < n) {
      decr();
      tmp = tmp + 1;
    }
  }

  /*+INFERRED@
    @ public normal_behavior
    @   ensures \result == this.value;
    @*/
  public int get() {
    return value;
  }
  
  //@ requires Integer.MIN_VALUE <= n && n <= Integer.MAX_VALUE;
  /*+INFERRED@
    @ public normal_behavior
    @   requires Integer.MIN_VALUE <= n && n <= Integer.MAX_VALUE;
    @   ensures this.value == n;
    @   assignable this.value;
    @*/
  public void set(int n) {
    value = n;
  }
  
}