// Annotations inferred with Strongarm with default settings in the VM provided by John Singleton.

public class ArrayList {
  public int size;
  public Object[] list;
  
  public ArrayList() {
    size = -1;
    list = new Object[10];
  }
  
  /*+INFERRED@
    @ public normal_behavior // (1) unnecessary
    @   ensures \result == this.size; // (1) correct
    @*/
  public int size() {
    return size;
  }
  
  /*+INFERRED@
    @ public normal_behavior // (1) correct
    @   requires (index <= this.size); // (1) correct
    @   requires 0 <= index; // (1) correct
    @   ensures \result == this.list[index]; // (1) correct
    @*/
  public Object get(int index) {
    if (0 <= index && index <= size) {
      return list[index];
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  /*+INFERRED@
    @ public normal_behavior // (1) unnecessary
    @   requires !(0 <= this.size); // (1) correct (part of postcondition where \result == false)
    @   ensures \result == false; // (1) correct
    @*/
  public boolean contains(Object o) {
    for (int i = 0; i <= size; i++) {
      if (list[i] == o) {
        return true;
      }
    }
    return false;
  }
  
  /*+INFERRED@
    @ public normal_behavior // (1) correct
    @   requires (index <= this.size); // (1) correct
    @   requires 0 <= index; // (1) correct
    @   ensures this.list[index] == o; // (1) correct
    @   assignable this.list[index]; // (1) correct
    @*/
  public void set(int index, Object o) {
    if (0 <= index && index <= size) {
      list[index] = o;
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  /*+INFERRED@
    @ private normal_behavior // (1) correct
    @   {|
    @     requires !(0 <= this.size); // (1) unnecessary
    @     requires this.list.length >= MAX_VALUE - 10; // (1) unnecessary
    @     assignable this.list; // (1) correct
    @     {|
    @       requires (this.list.length < MAX_VALUE); // (1) correct
    @       ensures this.list.length == MAX_VALUE; // (1) unnecessary
    @     also // (1) unnecessary
    @       requires !(this.list.length < MAX_VALUE); // (1) unnecessary
    @       ensures this.list.length == this.list.length + 10; // (1) unnecessary
    @     |}
    @   also // (1) unnecessary
    @     requires !(0 <= this.size); // (1) unnecessary
    @     requires !(this.list.length >= MAX_VALUE - 10); // (1) unnecessary
    @     ensures this.list.length == this.list.length + 10; // (1) unnecessary
    @     assignable this.list; // (1) correct
    @   |}
    @*/
  private void enlarge() {
    Object[] newList;
    if (list.length >= Integer.MAX_VALUE-10 && list.length < Integer.MAX_VALUE) {
      newList = new Object[Integer.MAX_VALUE];
    } else {
      newList = new Object[list.length + 10];
    }
    for (int i = 0; i <= size; i++) {
      newList[i] = list[i];
    }
    list = newList;
  }
  
  /*+INFERRED@
    @ public normal_behavior // (1) correct
    @   requires !(\old(THIS.size) == MAX_VALUE - 1); // (1) correct, only covers part of precondition
    @   ensures \old(this.list)[this.size] == o; // (1) correct
    @   ensures this.size == \old(this.size) + 1; // (1) correct
    @   assignable \old(this.list)[this.size]; // (1) correct, does not consider that it may modify 'list' when enlarge is called
    @   assignable this.size; // (1) correct
    @   {|
    @     requires (\old(THIS.size) == this.list.length - 1); // (1) unnecessary
    @   also (1) correct
    @     requires !(\old(THIS.size) == this.list.length - 1); // (1) unnecessary
    @   |}
    @*/
  public void add(Object o) {
    if (size == Integer.MAX_VALUE - 1) {
      throw new IndexOutOfBoundsException();
    }
    if (size == list.length - 1) {
      enlarge();
    }
    size = size + 1;
    list[size] = o;
  }
  
  public void remove(Object o) {
    for (int i = 0; i <= size; i++) {
      if (list[i] == o) {
        for (int j = i; j < size; j++) {
          list[j] = list[j+1];
        }
        size = size - 1;
        return;
      }
    }
  }
  
  /*+INFERRED@
    @ public normal_behavior // (1) unnecessary
    @   ensures \result == (this.size == -1); // (1) correct
    @*/
  public boolean isEmpty() {
    return size == -1;
  }
  
}