public class ArrayList {
  //@ public invariant -1 <= size <= Integer.MAX_VALUE;
  //@ public invariant list != null;
  //@ public invariant size < list.length;
  //@ public invariant (\forall int i; 0 <= i <= size; list[i] != null);
  public int size;
  //@ public invariant 0 <= list.length <= Integer.MAX_VALUE;
  public Object[] list;
  
  //@ ensures size == -1;
  //@ ensures \fresh(list);
  //@ pure
  public ArrayList() {
    size = -1;
    list = new Object[10];
  }
  
  //@ ensures \result == size;
  //@ pure
  public int size() {
    return size;
  }
  
  //@ public normal_behavior
  //@   requires 0 <= index && index <= size;
  //@   assignable \nothing;
  //@   ensures \result == list[index];
  //@ also
  //@ public exceptional_behavior
  //@  requires index < 0 || index > size;
  //@  assignable \nothing;
  //@  signals (IndexOutOfBoundsException) true;
  public Object get(int index) {
    if (0 <= index && index <= size) {
      return list[index];
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  //@ requires o != null;
  //@ ensures \result == false <==> size == -1 || (\forall int i; 0 <= i <= size; list[i] != o);
  //@ ensures \result == true <==> (\exists int i; 0 <= i <= size; list[i] == o);
  //@ pure
  public boolean contains(Object o) {
    //@ loop_invariant 0 <= i <= size+1;
    //@ loop_invariant (\forall int j; 0 <= j < i; list[j] != o);
    //@ decreases size - i;
    for (int i = 0; i <= size; i++) {
      if (list[i] == o) {
        return true;
      }
    }
    return false;
  }
  
  //@ public normal_behavior
  //@   requires 0 <= index <= size;
  //@   requires o != null;
  //@   requires \type(Object) <: \elemtype(\typeof(list)); // needed to prevent PossiblyBadArrayAssignment warning
  //@   assignable list[index];
  //@   ensures list[index] == o;
  //@ also
  //@ public exceptional_behavior
  //@   requires index < 0 || index > size;
  //@   assignable \nothing;
  //@   signals (IndexOutOfBoundsException) true;
  public void set(int index, Object o) {
    if (0 <= index && index <= size) {
      list[index] = o;
    } else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  //@ private normal_behavior
  //@   requires list.length < Integer.MAX_VALUE;
  //@   requires \type(Object) <: \elemtype(\typeof(list));
  //@   assignable list;
  //@   ensures \fresh(list);
  //@   ensures \type(Object) <: \elemtype(\typeof(list));
  //@   ensures (\forall int i; 0 <= i <= size; list[i] == \old(list[i]));
  //@   ensures list.length > \old(list.length);
  private void enlarge() {
    Object[] newList;
    if (list.length >= Integer.MAX_VALUE-10 && list.length < Integer.MAX_VALUE) {
      newList = new Object[Integer.MAX_VALUE];
    } else {
      newList = new Object[list.length + 10];
    }
    //@ loop_invariant 0 <= i <= size + 1;
    //@ loop_invariant (\forall int j; 0 <= j < i; newList[j] == \pre(list[j]));
    //@ decreases size - i;
    for (int i = 0; i <= size; i++) {
      newList[i] = list[i];
    }
    list = newList;
  }
  
  //@ public normal_behavior
  //@   requires size < Integer.MAX_VALUE - 1;
  //@   requires o != null;
  //@   requires \type(Object) <: \elemtype(\typeof(list));
  //@   assignable size, list, list[*];
  //@   ensures (\forall int i; 0 <= i <= \old(size); list[i] == \old(list[i]));
  //@   ensures size == \old(size) + 1;
  //@   ensures list[size] == o;
  //@ also
  //@ public exceptional_behavior
  //@   requires size == Integer.MAX_VALUE - 1;
  //@   assignable size;
  //@   signals (IndexOutOfBoundsException) true;
  public void add(Object o) {
    if (size == Integer.MAX_VALUE - 1) {
      throw new IndexOutOfBoundsException();
    }
    if (size == list.length - 1) {
      enlarge();
    }
    size = size + 1;
    list[size] = o;
  }
  
  //@ public normal_behavior
  //@   requires o != null && size >= 0;
  //@   requires (\exists int i; 0 <= i <= size; list[i] == o); // The element to remove is in the list
  //@   assignable list[*], size;
  //@   ensures size == \old(size) - 1;
  //@   ensures (\exists int i; 0 <= i <= \old(size); \old(list[i]) == o && (\forall int j; 0 <= j < i; list[j] == \old(list[j])) && (\forall int k; i <= k <= size; list[k] == \old(list[k+1])));
  //@ also
  //@ public normal_behavior
  //@   requires o != null && size >= 0;
  //@   requires (\forall int i; 0 <= i <= size; list[i] != o); // The element to remove is NOT in the list
  //@   assignable list[*], size;
  //@   ensures size == \old(size);
  //@   ensures (\forall int i; 0 <= i <= size; list[i] == \old(list[i]));
  //@ also
  //@ public normal_behavior
  //@   requires size == -1; // Empty list
  //@   assignable \nothing;
  public void remove(Object o) {
    
    //@ loop_invariant 0 <= i <= \old(size) + 1;
    //@ loop_invariant (\forall int k; 0 <= k <= size; list[k] == \old(list[k]));
    //@ loop_invariant size == \old(size);
    //@ loop_invariant (\forall int k; 0 <= k < i; list[k] != o);
    //@ loop_modifies list[*], size;
    //@ decreases size - i;
    for (int i = 0; i <= size; i++) {
      if (list[i] == o) {
        //@ loop_invariant i <= j <= size;
        //@ loop_invariant (\forall int k; 0 <= k < i; list[k] != o);
        //@ loop_invariant (\forall int k; 0 <= k < i; list[k] == \old(list[k]));
        //@ loop_invariant (\forall int k; i <= k < j; list[k] == \old(list[k+1]));
        //@ loop_invariant (\forall int k; j < k <= size; list[k] == \old(list[k]));
        //@ loop_modifies list[j..size];
        //@ decreases size - j;
        for (int j = i; j < size; j++) {
          list[j] = list[j+1];
        }
        size = size - 1;
        return;
      }
    }
  }
  
  //@ ensures \result <==> size == -1;
  //@ pure
  public boolean isEmpty() {
    return size == -1;
  }
  

}