// Verified with OpenJML-0.17.0-alpha-14, command: "./openjml --esc --progress path/to/Counter.java"
// OpenJML then shows that all method assertions were validated

public class Counter {
  public int value;

  //@ ensures this.value == 0;
  public Counter() {
    value = 0;
  }
  
  //@ requires Integer.MIN_VALUE <= initialValue && initialValue <= Integer.MAX_VALUE;
  //@ ensures this.value == initialValue;
  public Counter(int initialValue) {
    value = initialValue;
  }
  
  //@ requires Integer.MIN_VALUE <= this.value && this.value < Integer.MAX_VALUE;
  //@ assignable value;
  //@ ensures this.value == \old(this.value) + 1;
  public void incr() {
    value = value + 1;
  }
  
  //@ requires Integer.MIN_VALUE <= this.value && this.value <= Integer.MAX_VALUE - n;
  //@ requires n >= 0;
  //@ assignable this.value;
  //@ ensures this.value == \old(this.value) + n;
  public void incrByN(int n) {
    int tmp = n;
    //@ loop_invariant this.value == \old(this.value) + (n-tmp);
    //@ decreases tmp;
    while (tmp > 0) {
      incr();
      tmp = tmp - 1;
    }
  }
  
  
  //@ requires Integer.MIN_VALUE + 1 <= this.value && this.value <= Integer.MAX_VALUE;
  //@ assignable this.value;
  //@ ensures this.value == \old(this.value) - 1;
  public void decr() {
    value = value - 1;
  }
  
  //@ requires Integer.MIN_VALUE + n <= this.value && this.value <= Integer.MAX_VALUE;
  //@ requires n >= 0;
  //@ assignable this.value;
  //@ ensures this.value == \old(this.value) - n;
  public void decrByN(int n) {
    int tmp = 0;
    //@ loop_invariant this.value == \old(this.value) - tmp;
    //@ decreases n-tmp;
    while (tmp < n) {
      decr();
      tmp = tmp + 1;
    }
  }

  //@ ensures \result == this.value;
  //@ pure
  public int get() {
    return value;
  }
  
  //@ requires Integer.MIN_VALUE <= n && n <= Integer.MAX_VALUE;
  //@ assignable this.value;
  //@ ensures this.value == n;
  public void set(int n) {
    value = n;
  }
  
}